module FeedBo
  class Robot
    
    include Drawable
    include Transformable
  
    attr_accessor :level
    attr_reader :tools
  
    def initialize
      super
      @tools = []
    end
  
    def draw
      s = Level::FIELD_SIZE
      @svg.image width: s, height: s, :'xlink:href' => h.image_path("robot.svg")
            
      @tools.each do |tool|
        tool.draw(@svg)
      end
    end
    
    def setup
    end
  
    def run
    end
  
    def continue(last_activity)
      until @level.finish?
        if last_activity == :move
          move
        else
          @level.perform_step
        end
      end
    end
    
    def add_tool(tool)
      tool.robot = self
      @tools << tool
    end
    
    def hit_positions
      positions = [Level.vector_sum(@position, movement)]
      if @direction.odd?
        positions << Level.vector_sum(@position, [movement[0],0])
        positions << Level.vector_sum(@position, [0,movement[1]])
      end
      return positions
    end
  
    protected
  
    def move(steps=1)
      steps.times do
        if hit_positions.none? { |p| @level.position_blocked? p }
          @level.elements.select { |e| hit_positions.include? e.position }.each do |e|
            e.hit if e.respond_to? :hit
          end
          @position = Level.vector_sum(@position, movement)
          animate_move
        end
        @level.perform_step
      end
      return :move
    end
  
    def movement
      x = case @direction
        when 0,4 then 0
        when 1,2,3 then 1
        when 5,6,7 then -1
      end
      y = case @direction
        when 2,6 then 0
        when 3,4,5 then 1
        when 7,0,1 then -1
      end
      return [x, y]
    end
  
    def animate_move
      @svg.animateMotion values: "0,0; #{movement[0]*Level::FIELD_SIZE},#{movement[1]*Level::FIELD_SIZE}", 
        begin: "#{@level.step*Level::STEP_DUR}s", dur: "#{Level::STEP_DUR}s", additive: 'sum', fill: 'freeze'
    end
  
    def turn(steps)
      step = steps > 0 ? 1 : -1
      steps.abs.times do
        return false if @level.finish?
        @direction += step
        @direction = @direction % 8
        animate_turn(step)
        @level.perform_step
      end
    end
  
    def turn_right(steps=1)
      turn steps
    end
  
    def turn_left(steps=1)
      turn -1 * steps
    end
  
    def animate_turn(step)
      s = Level::FIELD_SIZE
      @svg.animateTransform attributeName: 'transform', type: 'rotate', values: "0,#{s/2},#{s/2}; #{step*45},#{s/2},#{s/2}", 
        begin: "#{@level.step*Level::STEP_DUR}s", dur: "#{Level::STEP_DUR}s", additive: 'sum', fill: 'freeze'
    end
    
    def wait(steps=1)
      steps.times do
        @level.perform_step
      end
    end
  
  end
end